# -*- Mode: cperl -*-
#--------------------------------------------------------------------
# Copyright (C) 2000, 2001 by MandrakeSoft.
# Chmouel Boudjnah <chmouel@mandrakesoft.com>.
#
# Redistribution of this file is permitted under the terms of the GNU 
# Public License (GPL)
#--------------------------------------------------------------------
# $Id: common.pm,v 1.12 2001/04/27 14:27:33 chmouel Exp $
#--------------------------------------------------------------------

package common;
use strict;

my $debug = 0;

my $sysconfig = $debug ? "./installkernel.sysconfig" : "/etc/sysconfig/installkernel";

sub check_default_entry {
    # dont ask me why i need to do that :p
    my $foo = $_[0];
    my %entry = %$foo;
    my $foo1 = $_[1];
    my %main = %$foo1;
    
    # check if the default entry if there is an initrd pointing to the
    # right kernel.
    while (my $act = each %entry) {
	
	# find the default ?
	if ($act =~ /^$main{default}$/) {
	    
	    # there is an initrd ?
	    if ($entry{$act}{initrd}) {
		
		my $initrd_version_of_entry;
		my $kernel_version_of_entry;	    
		
		#too much code :p but i see problems coming and more
		#different code between the two case
		#ps: the smp/enterprise hack is very bad :p
		my $initrd_of_entry = $entry{$act}{initrd};
		
		if ( -l $initrd_of_entry ) {
		    
		    my $initrd_of_entry_link = readlink ($initrd_of_entry);
		    if ($initrd_of_entry_link =~ m,/boot/initrd-(\d+.*mdk(\.|smp\.|enterprise\.|secure\.))img,) {
			($initrd_version_of_entry=$1) =~ s|\.$||;
		    }
		} elsif ( $initrd_of_entry =~ m,/boot/initrd-(\d+.*mdk(\.|smp\.|enterprise\.|secure\.))img,) {
		    ($initrd_version_of_entry=$1) =~ s|\.$||;
		}

		my $kernel_of_entry = $entry{$act}{kernel};
		if ( -l $kernel_of_entry ) {
		    my $kernel_of_entry_link = readlink($kernel_of_entry);
		    
		    if ($kernel_of_entry_link =~ /vmlinuz-(\d+.*mdk($|smp|enterprise|secure))/) {
			$kernel_version_of_entry=$1;
		    }
		} elsif ( $kernel_of_entry =~ /vmlinuz-(\d+.*mdk($|smp|enterprise|secure))/) {
		    $kernel_version_of_entry=$1;
		}
		
		if ($kernel_version_of_entry !~ /^$initrd_version_of_entry$/) {
		    print STDERR ("look like there was a problem, the default vmlinuz version is not the same \nof the initrd\n");
		    exit(1);
		}
	    }
	}
     }
}

sub get_first_boot {
    my ($type, $resolved_link) = @_;
    my $boot = '/boot/';
    my @list;
    opendir D, $boot;
    for (readdir(D)) { 
	if (/vmlinuz-.*$/i and not /$resolved_link/ and not /vmlinuz-(smp|enterprise|secure)/) {
	    push @list, $_ 
	}
    }
    close D;
    @list = reverse(sort(@list));
    ( my $first_kernel = shift(@list) ) =~ s|vmlinuz-||;
    system("perl /usr/share/loader/make-initrd $first_kernel") unless $debug;
    return $first_kernel;
}

sub do_check {
    my ($label, $version) = @_;
    if ($label) {
	print("not adding entry, $label already exists\n");
	exit;
    }
    unless (-f "/boot/vmlinuz-$version") {
	print("look like i cannot find the specified kernel /boot/vmlinuz-$version\n") ;
	exit;
    }
}

sub symlinkf { unlink $_[1]; symlink $_[0], $_[1] }
sub cat_ { local *F; open F, $_[0] or $_[1] ? die "cat of file $_[0] failed: $!\n" : return; my @l = <F>; wantarray ? @l : join '', @l }

sub getoptions {
    my $lilo = shift;
    my ($options, $foo);
    my %var = getVarsFromSh("$sysconfig");
    my $cmdline = grep(/NOCMDLINE/, $var{OPTIONS}) ? undef : cat_("/proc/cmdline");chomp $cmdline;
    foreach $foo (split(/\s+/, join(" ", $var{OPTIONS}, $cmdline)))  {
	next if $foo =~ /auto/ and $lilo;
	next if $foo =~ /root=/ and $lilo;
	next if $foo =~ /^r[ow]/;
	next if $foo =~ /BOOT_IMAGE=/;
	next if $foo =~ /NOCMDLINE/;
	next if (grep /$foo/, $options);
	$options .= "$foo ";
    }
    return $options;
}

sub getroot {
    my $lilo = shift;
    my $root;
    my @r = grep /root=\/\S+/, split(" ", cat_('/proc/cmdline'));$root = $r[0];
    $root = getroot_from_fstab() unless $root;
    ($root = `rdev`) =~ s|(\S+)\s+.*$|root=$1| unless $root;chomp $root;
    warn("There is no root= defined you are going in trouble\n") unless $root;
    $root =~ s|root=|| if $lilo;
    return $root;
}

sub getroot_from_fstab {
    my $part;
    local *F;
    open F, '/etc/fstab';
    while (<F>) {
	my @s = split ' ';
	$part = $s[0] if $s[1] =~ m|/$| and not $part;
    }
    close F;
    return $part;
}	
   

sub getVarsFromSh(%) {
    my %l;
    local *F; open F, $_[0] or return;
    local $_;
    while (<F>) {
	s/#.*//; # remove comments
	my ($v, $val, $val2) =
	  /^\s*			# leading space
	   (\w+) =		# variable
	   (
   	       "([^"]*)"	# double-quoted text
   	     | '([^']*)'	# single-quoted text
   	     | [^'"\s]+		# normal text
           )
           \s*$			# end of line
          /x or next;
	$l{$v} = defined $val2 ? $val2 : $val;
    }
    %l;
}

sub get_kernel_type {
    $_ = shift;
    if (/smp$/ ) {
	return "smp";
    } elsif (/secure$/) {
	return "secure";
    } elsif (/enterprise$/) {
	return "enterprise";
    }
    return undef;
}

1;#wonderful perl
